@@ -1,3 +1,4 @@
+import os
 from pathlib import Path
 from time import sleep
 
@@ -12,21 +13,29 @@ class TemplateCorpusTest:
     corp_normal_obj = None
     corp_kwic_obj = None
 
+    full_query = {
+        'ты': {
+            'gramm': {
+                'case': ['acc', 'nom', 'gen'],
+                'num': ['sg', 'pl']
+            },
+            'flags': {
+                'position': ['amark', 'bmark'],
+            }
+        }
+    }
+
     #########################
     #    Test requesting    #
     #########################
 
     def test_empty_query(self):
         with pytest.raises(ValueError):
-            corp = self.corp_type(
-                '', 1, marker=str.upper)
-            corp.request_examples()
+            self.corp_type('', 1, marker=str.upper)
 
     def test_p_count_zero(self):
         with pytest.raises(ValueError):
-            corp = self.corp_type(
-                'корпус', 0, marker=None)
-            corp.request_examples()
+            self.corp_type('корпус', 0, marker=None)
 
     def test_one_str_with_one_word(self):
         corp = self.corp_type('ты', 1, marker=str.capitalize, spd=1, dpp=5)
@@ -43,7 +52,7 @@ class TemplateCorpusTest:
         sleep(5)
 
     def test_one_form_with_one_gram(self):
-        corp = self.corp_type({'ты': 'nom'}, 1, marker=str.capitalize, spd=1)
+        corp = self.corp_type({'ты': {'gramm': 'nom'}}, 1, marker=str.capitalize, spd=1)
         corp.request_examples()
 
         assert len(corp) > 1
@@ -63,18 +72,7 @@ class TemplateCorpusTest:
         sleep(5)
 
     def test_one_form_with_several_grams_and_flags(self):
-        query = {
-            'ты': {
-                'gramm': {
-                    'case': ['acc', 'nom', 'gen'],
-                    'num': ['sg', 'pl']
-                },
-                'flags': {
-                    'position': ['amark', 'bmark'],
-                }
-            }
-        }
-        corp = self.corp_type(query, 1, marker=str.capitalize)
+        corp = self.corp_type(self.full_query, 1, marker=str.capitalize)
         corp.request_examples()
 
         assert len(corp) > 1
@@ -88,36 +86,14 @@ class TemplateCorpusTest:
         sleep(5)
 
     def test_full_query_with_kwic_without_kwsz(self):
-        query = {
-            'ты': {
-                'gramm': {
-                    'case': ['acc', 'nom', 'gen'],
-                    'num': ['sg', 'pl']
-                },
-                'flags': {
-                    'position': ['amark', 'bmark'],
-                }
-            }
-        }
-        corp = self.corp_type(query, 1, marker=str.capitalize, out='kwic')
+        corp = self.corp_type(self.full_query, 1, marker=str.capitalize, out='kwic')
         corp.request_examples()
 
         assert len(corp) > 1
         sleep(5)
 
     def test_full_query_with_kwic_with_kwsz(self):
-        query = {
-            'ты': {
-                'gramm': {
-                    'case': ['acc', 'nom', 'gen'],
-                    'num': ['sg', 'pl']
-                },
-                'flags': {
-                    'position': ['amark', 'bmark'],
-                }
-            }
-        }
-        corp = self.corp_type(query, 1, marker=str.capitalize, out='kwic', kwsz=7)
+        corp = self.corp_type(self.full_query, 1, marker=str.capitalize, out='kwic', kwsz=7)
         corp.request_examples()
 
         assert len(corp) > 1
@@ -134,13 +110,15 @@ class TemplateCorpusTest:
 
         assert default.data != by_creation_date.data
 
-    def test_subcorpus(self):
+    def test_mycorp(self):
         corp = self.corp_type(
             'ты', 1,
-            subcorpus="JSONeyJkb2Nfc2V4IjogWyLQvNGD0LYiXSwgImRvY19pX3RhZ2dpbmciOiBbIjEiXX0%3D"
+            mycorp="JSONeyJkb2Nfc2V4IjogWyLQvNGD0LYiXSwgImRvY19pX3RhZ2dpbmciOiBbIjEiXX0%3D"
         )
         corp.request_examples()
+
         assert len(corp) >= 1
+        sleep(5)
 
     def test_call(self):
         corp = self.corp_type('ты', 1, marker=str.capitalize, out='kwic')
@@ -178,17 +156,13 @@ class TemplateCorpusTest:
 
     def test_load_to_wrong_corpus(self):
         with pytest.raises(NotImplementedError):
-            self.corp_type(file='data\\wrong_mode.csv')
+            self.corp_type(file=f'data{os.sep}wrong_mode.csv')
 
     def test_request_if_base_loaded(self):
         corp = self.corp_type(file=self.corp_normal_obj.file)
         with pytest.raises(RuntimeError):
             corp.request_examples()
 
-    def test_wrong_filetype(self):
-        with pytest.raises(TypeError):
-            corp = self.corp_type('ты', 1, file='file.txt')
-
     def test_default_filetype(self):
         corp = self.corp_type('ты', 1)
 
@@ -216,7 +190,7 @@ class TemplateCorpusTest:
         )
 
     def test_query_dict(self):
-        corp = self.corp_type({'ты': 'acc'}, 1)
+        corp = self.corp_type({'ты': {'gramm': 'acc'}}, 1)
 
         assert isinstance(corp.query, dict) and corp.query
 
@@ -226,7 +200,7 @@ class TemplateCorpusTest:
         assert isinstance(corp.query, str) and corp.query
 
     def test_forms_in_query_dict(self):
-        corp = self.corp_type({'ты': 'acc', 'готов': {}}, 1)
+        corp = self.corp_type({'ты': {'gramm': 'acc'}, 'готов': {}}, 1)
 
         assert isinstance(corp.forms_in_query, list)
         assert all(isinstance(form, str) for form in corp.forms_in_query)
@@ -256,6 +230,18 @@ class TemplateCorpusTest:
 
         assert isinstance(corp.url, str) and corp.url
 
+    def test_amount_of_docs_normal(self):
+        assert isinstance(self.corp_normal_obj.amount_of_docs, int)
+
+    def test_amount_of_docs_kwic(self):
+        assert self.corp_kwic_obj.amount_of_docs is None
+
+    def test_amount_of_contexts_normal(self):
+        assert isinstance(self.corp_normal_obj.amount_of_contexts, int)
+
+    def test_amount_of_contexts_kwic(self):
+        assert isinstance(self.corp_kwic_obj.amount_of_contexts, int)
+
     ##########################
     # Test working with data #
     ##########################
@@ -263,6 +249,9 @@ class TemplateCorpusTest:
     def test_open_url(self):
         self.corp_normal_obj.open_url()
 
+    def test_open_graphic(self):
+        self.corp_normal_obj.open_graphic()
+
     def test_copy(self):
         copy = self.corp_normal_obj.copy()
 
@@ -339,70 +328,7 @@ class TemplateCorpusTest:
         copy = self.corp_normal_obj.copy()
         del copy[:]
 
-        assert copy == 0
-
-    ##########################
-    #        Test <=>        #
-    ##########################
-
-    def test_lt_with_int(self):
-        assert self.corp_normal_obj < 10
-        assert self.corp_kwic_obj < 10
-
-    def test_lt_with_corp(self):
-        copy = self.corp_normal_obj.copy()
-        del copy[0]
-
-        assert copy < self.corp_normal_obj
-
-    def test_le_with_int(self):
-        assert self.corp_normal_obj <= 5
-        assert self.corp_kwic_obj <= 5
-
-    def test_le_with_corp(self):
-        copy = self.corp_normal_obj.copy()
-        del copy[0]
-
-        assert copy <= self.corp_normal_obj
-
-    def test_eq_with_int(self):
-        assert self.corp_normal_obj == len(self.corp_normal_obj)
-        assert self.corp_kwic_obj == len(self.corp_kwic_obj)
-
-    def test_eq_with_corp(self):
-        copy = self.corp_normal_obj.copy()
-
-        assert copy == self.corp_normal_obj
-
-    def test_ne_with_int(self):
-        assert self.corp_normal_obj != 0
-        assert self.corp_kwic_obj != 0
-
-    def test_ne_with_corp(self):
-        copy = self.corp_normal_obj.copy()
-        del copy[0]
-
-        assert copy != self.corp_normal_obj
-
-    def test_gt_with_int(self):
-        assert self.corp_normal_obj > 0
-        assert self.corp_kwic_obj > 0
-
-    def test_gt_with_corp(self):
-        copy = self.corp_normal_obj.copy()
-        del copy[0]
-
-        assert self.corp_normal_obj > copy
-
-    def test_ge_with_int(self):
-        assert self.corp_normal_obj >= 1
-        assert self.corp_kwic_obj >= 1
-
-    def test_ge_with_corp(self):
-        copy = self.corp_normal_obj.copy()
-        del copy[0]
-
-        assert self.corp_normal_obj >= copy
+        assert len(copy.data) == 0
 
     ##########################
     #   Test class setters   #
@@ -415,7 +341,7 @@ class TemplateCorpusTest:
         assert corp.spd is 20
 
     def test_set_spd_exception(self):
-        with pytest.raises(TypeError):
+        with pytest.raises(ValueError):
             self.corp_type.set_spd('12')
 
     def test_set_dpp_normal(self):
@@ -425,7 +351,7 @@ class TemplateCorpusTest:
         assert corp.dpp is 20
 
     def test_set_dpp_exception(self):
-        with pytest.raises(TypeError):
+        with pytest.raises(ValueError):
             self.corp_type.set_dpp('12')
 
     def test_set_text_normal(self):
@@ -435,7 +361,7 @@ class TemplateCorpusTest:
         assert corp.text == 'lexform'
 
     def test_set_text_exception(self):
-        with pytest.raises(TypeError):
+        with pytest.raises(ValueError):
             self.corp_type.set_text(12)
 
     def test_set_sort_normal(self):
@@ -445,7 +371,7 @@ class TemplateCorpusTest:
         assert corp.sort == 'i_grcreated_inv'
 
     def test_set_sort_exception(self):
-        with pytest.raises(TypeError):
+        with pytest.raises(ValueError):
             self.corp_type.set_sort(12)
 
     def test_set_min_normal(self):
@@ -455,7 +381,7 @@ class TemplateCorpusTest:
         assert corp.min2 is 10
 
     def test_set_min_exception(self):
-        with pytest.raises(TypeError):
+        with pytest.raises(ValueError):
             self.corp_type.set_min('12')
 
     def test_set_max_normal(self):
@@ -465,7 +391,7 @@ class TemplateCorpusTest:
         assert corp.max2 is 10
 
     def test_set_max_exception(self):
-        with pytest.raises(TypeError):
+        with pytest.raises(ValueError):
             self.corp_type.set_max('12')
 
     def test_set_restrict_show_exception_str(self):
@@ -501,13 +427,19 @@ class TestPaper2000Corpus(TemplateCorpusTest):
     corp_kwic_obj.request_examples()
     sleep(5)
 
-    def test_subcorpus(self):
+    def test_mycorp(self):
         corp = self.corp_type(
             'ты', 1,
-            subcorpus="JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDEwIl19"
+            mycorp="JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDEwIl19"
         )
         corp.request_examples()
+
         assert len(corp) >= 1
+        sleep(5)
+
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
 
 
 class TestPaperRegionalCorpus(TemplateCorpusTest):
@@ -528,13 +460,19 @@ class TestPaperRegionalCorpus(TemplateCorpusTest):
         assert len(corp) > 1
         sleep(5)
 
-    def test_subcorpus(self):
+    def test_mycorp(self):
         corp = self.corp_type(
             'ты', 1,
-            subcorpus="JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDEwIl19"
+            mycorp="JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDEwIl19"
         )
         corp.request_examples()
+
         assert len(corp) >= 1
+        sleep(5)
+
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
 
 
 class TestParallelCorpus(TemplateCorpusTest):
@@ -555,13 +493,15 @@ class TestParallelCorpus(TemplateCorpusTest):
         assert len(corp) > 1
         sleep(5)
 
-    def test_subcorpus(self):
+    def test_mycorp(self):
         corp = self.corp_type(
             'ты', 1,
-            subcorpus='JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDAwIl19'
+            mycorp='JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDAwIl19'
         )
         corp.request_examples()
+
         assert len(corp) >= 1
+        sleep(5)
 
     def test_sort_data(self):
         copy = self.corp_normal_obj.copy()
@@ -588,16 +528,18 @@ class TestMultilingualParaCorpus(TemplateCorpusTest):
         assert len(corp) > 1
         sleep(5)
 
-    def test_subcorpus(self):
+    def test_mycorp(self):
         pass
-        # there's a bug on RNC
+        # it's impossible to set subcorpus in MultilingualParaCorpus
 
         # corp = self.corp_type(
         #     'ты', 1,
-        #     subcorpus='JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDAwIl19'
+        #     mycorp='JSONeyJkb2NfaV9sZV9zdGFydF95ZWFyIjogWyIyMDAwIl19'
         # )
         # corp.request_examples()
+        #
         # assert len(corp) >= 1
+        # sleep(5)
 
     def test_sort_data(self):
         copy = self.corp_normal_obj.copy()
@@ -631,6 +573,10 @@ class TestMultilingualParaCorpus(TemplateCorpusTest):
     def test_found_wordforms_from_file(self):
         pass
 
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
+
 
 class TestTutoringCorpus(TemplateCorpusTest):
     corp_type = rnc.TutoringCorpus
@@ -650,6 +596,10 @@ class TestTutoringCorpus(TemplateCorpusTest):
         assert len(corp) >= 1
         sleep(5)
 
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
+
 
 class TestDialectalCorpus(TemplateCorpusTest):
     corp_type = rnc.DialectalCorpus
@@ -669,16 +619,19 @@ class TestDialectalCorpus(TemplateCorpusTest):
         assert len(corp) > 1
         sleep(5)
 
-    def test_subcorpus(self):
+    def test_mycorp(self):
         corp = self.corp_type(
             'ты', 1,
-            subcorpus='JSONeyJkb2NfcmVnaW9uIjogWyLQmtCw0YDQtdC70LjRjyJdfQ=='
+            mycorp='JSONeyJkb2NfcmVnaW9uIjogWyLQmtCw0YDQtdC70LjRjyJdfQ%3D%3D'
         )
-        # TODO: написать преобразование URL символов в обычные, поскольку
-        #  aiohttp переводит символы в нужный код сам, а уже переведённые
-        #  переводит ещё раз
         corp.request_examples()
+
         assert len(corp) >= 1
+        sleep(5)
+
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
 
 
 class TestSpokenCorpus(TemplateCorpusTest):
@@ -692,6 +645,10 @@ class TestSpokenCorpus(TemplateCorpusTest):
     corp_kwic_obj.request_examples()
     sleep(5)
 
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
+
 
 class TestAccentologicalCorpus(TemplateCorpusTest):
     corp_type = rnc.AccentologicalCorpus
@@ -704,11 +661,50 @@ class TestAccentologicalCorpus(TemplateCorpusTest):
     corp_kwic_obj.request_examples()
     sleep(5)
 
-    def test_subcorpus(self):
+    def test_mycorp(self):
         corp = self.corp_type(
             'ты', 1,
-            subcorpus='JSONeyJkb2NfYXV0aG9yIjogWyLQkC7QoS4g0J_Rg9GI0LrQuNC9Il0sICJkb2NfaV9sZV9lbmRfeWVhciI6IFsiMTgzMCJdfQ=='
+            mycorp='JSONeyJkb2NfYXV0aG9yIjogWyLQkC7QoS4g0J_Rg9GI0LrQuNC9Il0sICJkb2NfaV9sZV9lbmRfeWVhciI6IFsiMTgzMCJdfQ%3D%3D'
+        )
+        corp.request_examples()
+
+        assert len(corp) >= 1
+        sleep(5)
+
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
+
+
+class TestMultimodalCorpus(TemplateCorpusTest):
+    corp_type = rnc.MultimodalCorpus
+
+    corp_normal_obj = corp_type('корпус', 1, dpp=5, spd=1)
+    corp_kwic_obj = corp_type('корпус', 1, dpp=5, spd=1, out='kwic')
+
+    corp_normal_obj.request_examples()
+    sleep(5)
+    corp_kwic_obj.request_examples()
+    sleep(5)
+
+    def test_download_all(self):
+        self.corp_normal_obj.download_all()
+        files = os.listdir(self.corp_normal_obj.MEDIA_FOLDER)
+
+        assert all(
+            ex.filepath.name in files
+            for ex in self.corp_normal_obj
         )
 
+    def test_mycorp(self):
+        pass
+
+    def test_open_graphic(self):
+        with pytest.raises(RuntimeError):
+            self.corp_normal_obj.open_graphic()
+
+    def test_full_query_with_lexform(self):
+        corp = self.corp_type('ты это', 1, text='lexform')
         corp.request_examples()
-        assert len(corp) >= 1
\ No newline at end of file
+
+        assert len(corp.data) > 1
